package com.jpsoft.picc.modules.auth.controller;

import cn.hutool.core.convert.Convert;
import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyMember;
import com.jpsoft.picc.modules.base.entity.Jobs;
import com.jpsoft.picc.modules.base.service.CompanyMemberService;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.JobsService;
import com.jpsoft.picc.modules.business.entity.InsurancePolicy;
import com.jpsoft.picc.modules.business.service.InsurancePolicyService;
import com.jpsoft.picc.modules.common.config.OSSConfig;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.CheckIdCard;
import com.jpsoft.picc.modules.common.utils.OSSUtil;
import com.jpsoft.picc.modules.common.utils.POIUtils;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.sys.entity.DataDictionary;
import com.jpsoft.picc.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

@CrossOrigin
@Api(description="企业人员管理")
@RestController
@RequestMapping("/auth/companyMember")
public class CompanyMemberController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyMemberService companyMemberService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private JobsService jobsService;

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private OSSConfig ossConfig;

    @ApiOperation(value="读取企业人员列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<CompanyMember>> list(
                                   HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<List<CompanyMember>> msgResult = new MessageResult<>();

        try {
            if (company==null){
                throw new Exception("当前用户未填写企业信息!");
            }

            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("delFlag", false);

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "asc"));

            searchParams.put("companyId", company.getId());

            Page<CompanyMember> page = companyMemberService.pageSearch(searchParams, 1, 1000, sortList);

            msgResult.setResult(true);
            msgResult.setData(page.getResult());
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @ApiOperation(value="读取企业人员分页列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "memberName",value = "人员姓名", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "jobsId",value = "岗位编号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "excludePolicyId",value = "要排除投保单", required = false, paramType = "form",dataType = "String"),
    })
    public MessageResult<Map> pageList(
            @RequestParam(value="memberName",defaultValue="") String memberName,
            @RequestParam(value="jobsId",defaultValue="") String jobsId,
            @RequestParam(value="excludePolicyId",defaultValue="") String excludePolicyId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            if (company == null) {
                throw new Exception("当前用户未填写企业信息!");
            }

            Map<String, Object> searchParams = new HashMap<>();
            searchParams.put("delFlag", false);

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "asc"));

            searchParams.put("companyId", company.getId());

            if(StringUtils.isNotEmpty(jobsId)) {
                searchParams.put("jobsId", jobsId);
            }

            if(StringUtils.isNotEmpty(excludePolicyId)) {
                //todo 新增和被替换的投保单编号一致
                InsurancePolicy insurancePolicy = insurancePolicyService.get(excludePolicyId);
                searchParams.put("excludePolicyNo", insurancePolicy.getPolicyNo());
            }

            if(StringUtils.isNotEmpty(memberName)){
                searchParams.put("memberName", "%" + memberName + "%");
            }

            Page<CompanyMember> page = companyMemberService.pageSearch(searchParams, pageIndex, pageSize, sortList);

            msgResult.setResult(true);
            msgResult.setData(PojoUtils.pageWrapper(page));
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @ApiOperation(value="添加企业人员")
    @PostMapping("add")
    @ApiImplicitParams({
//            @ApiImplicitParam(name = "companyId",value = "企业ID", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "name",value = "姓名", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "cardType",value = "证件类型", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "cardNo",value = "证件编号", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "sex",value = "性别(男:true,女:false)", required = true, paramType = "form",dataType = "Boolean"),
            @ApiImplicitParam(name = "age",value = "年龄", required = true, paramType = "form",dataType = "int"),
            @ApiImplicitParam(name = "jobsId",value = "行业ID", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "healthStatus",value = "健康状态", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "cardFiles",value = "身份证件附件", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<CompanyMember> add(//@RequestParam(value="companyId",defaultValue="") String companyId,
                                            @RequestParam(value="name",defaultValue="") String name,
                                            @RequestParam(value="cardType",defaultValue="") String cardType,
                                            @RequestParam(value="cardNo",defaultValue="") String cardNo,
                                            @RequestParam(value="sex",defaultValue="null") Boolean sex,
                                            @RequestParam(value="age",defaultValue="0") Integer age,
                                            @RequestParam(value="jobsId",defaultValue="") String jobsId,
                                            @RequestParam(value="healthStatus",defaultValue="") String healthStatus,
                                            @RequestParam(value="cardFiles",defaultValue="") String cardFiles,
                                            HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<CompanyMember> msgResult = new MessageResult<>();
        CompanyMember companyMember = new CompanyMember();

        try {
            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("companyId",company.getId());
            searchParams.put("cardNo",cardNo);
            searchParams.put("delFlag",false);
            List<Sort> sortList = new ArrayList<>();
            Page<CompanyMember> page = companyMemberService.pageSearch(searchParams,1,100,sortList);
            if(page.size() > 0){
                msgResult.setResult(false);
                msgResult.setMessage("该身份证已经注册");
            }else {
                companyMember.setId(UUID.randomUUID().toString());
                if (StringUtils.isNotEmpty(company.getId())) companyMember.setCompanyId(company.getId());
                if (StringUtils.isNotEmpty(name)) companyMember.setName(name);
                if (StringUtils.isNotEmpty(cardType)) companyMember.setCardType(cardType);
                if (StringUtils.isNotEmpty(cardNo)) companyMember.setCardNo(cardNo);
                if (sex != null) companyMember.setSex(sex);
                if (age != 0) companyMember.setAge(age);
                if (StringUtils.isNotEmpty(jobsId)) companyMember.setJobsId(jobsId);
                if (StringUtils.isNotEmpty(healthStatus)) companyMember.setHealthStatus(healthStatus);
                if (StringUtils.isNotEmpty(cardFiles)) companyMember.setCardFiles(cardFiles);
//            if(StringUtils.isNotEmpty(status)) companyMember.setStatus(status);
                companyMember.setDelFlag(false);
                companyMember.setCreateBy(principal.getName());
                companyMember.setCreateTime(new Date());

                int affectCount = companyMemberService.insert(companyMember);

                if (affectCount > 0) {
                    msgResult.setResult(true);
                    msgResult.setData(companyMember);
                } else {
                    msgResult.setResult(false);
                    msgResult.setMessage("数据库添加失败");
                }
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取企业人员")
    @RequestMapping(value = "detail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<CompanyMember> detail(@RequestParam(value="id",defaultValue="") String id){
        MessageResult<CompanyMember> msgResult = new MessageResult<>();

        try {
            CompanyMember companyMember = companyMemberService.get(id);

            if (companyMember != null) {
                msgResult.setResult(true);
                msgResult.setData(companyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新企业人员")
    @PostMapping("update")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "name",value = "姓名", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "cardType",value = "证件类型", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "cardNo",value = "证件编号", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "sex",value = "性别(男:true,女:false)", required = true, paramType = "form",dataType = "Boolean"),
            @ApiImplicitParam(name = "age",value = "年龄", required = true, paramType = "form",dataType = "int"),
            @ApiImplicitParam(name = "jobsId",value = "行业ID", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "healthStatus",value = "健康状态", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "cardFiles",value = "身份证件附件", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<CompanyMember> update(@RequestParam(value="id",defaultValue="") String id,
//                                               @RequestParam(value="companyId",defaultValue="") String companyId,
                                               @RequestParam(value="name",defaultValue="") String name,
                                               @RequestParam(value="cardType",defaultValue="") String cardType,
                                               @RequestParam(value="cardNo",defaultValue="") String cardNo,
                                               @RequestParam(value="sex",defaultValue="null") Boolean sex,
                                               @RequestParam(value="age",defaultValue="0") Integer age,
                                               @RequestParam(value="jobsId",defaultValue="") String jobsId,
                                               @RequestParam(value="healthStatus",defaultValue="") String healthStatus,
                                               @RequestParam(value="cardFiles",defaultValue="") String cardFiles,
                                               HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<CompanyMember> msgResult = new MessageResult<>();

        try {
            CompanyMember companyMember = companyMemberService.get(id);

            if(StringUtils.isNotEmpty(company.getId())) companyMember.setCompanyId(company.getId());
            if(StringUtils.isNotEmpty(name)) companyMember.setName(name);
            if(StringUtils.isNotEmpty(cardType)) companyMember.setCardType(cardType);
            if(StringUtils.isNotEmpty(cardNo)) companyMember.setCardNo(cardNo);
            if(sex != null) companyMember.setSex(sex);
            if(age != 0) companyMember.setAge(age);
            if(StringUtils.isNotEmpty(jobsId)) companyMember.setJobsId(jobsId);
            if(StringUtils.isNotEmpty(healthStatus)) companyMember.setHealthStatus(healthStatus);
            if(StringUtils.isNotEmpty(cardFiles)) companyMember.setCardFiles(cardFiles);
//            if(StringUtils.isNotEmpty(status)) companyMember.setStatus(status);
            companyMember.setUpdateBy(principal.getName());
            companyMember.setUpdateTime(new Date());

            int affectCount = companyMemberService.update(companyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(companyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除企业人员")
    @PostMapping("delete")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Integer> delete(@RequestParam(value="id",defaultValue="") String id,
                                         HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            CompanyMember companyMember = companyMemberService.get(id);
            companyMember.setDelFlag(true);
            companyMember.setUpdateBy(principal.getName());
            companyMember.setUpdateTime(new Date());

            int affectCount = companyMemberService.update(companyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="导入企业人员")
    @PostMapping("importXls")
    @ApiImplicitParams({
//            @ApiImplicitParam(name = "companyId",value = "企业ID", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> importXls(//@RequestParam(value="companyId",defaultValue="") String companyId,
                                           MultipartFile uploadFile,
                                           HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<String> msgResult = new MessageResult<>();
        CompanyMember companyMember = new CompanyMember();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 7;

            List<DataDictionary> ddList = dataDictionaryService.findByCatalogName("证件类型");

            for(int rowIndex=1 ; rowIndex<sheet1.getLastRowNum(); rowIndex++){
                try {
                    String name = (String)poiUtils.getCellValue(sheetIndex,rowIndex,1);
                    String cardType = (String)poiUtils.getCellValue(sheetIndex,rowIndex,2);
                    String cardNo = (String)poiUtils.getCellValue(sheetIndex,rowIndex,3);
                    String sex = (String)poiUtils.getCellValue(sheetIndex,rowIndex,4);
//                  String age = array[5].toString();
                    String jobName = (String)poiUtils.getCellValue(sheetIndex,rowIndex,5);
                    String healthStatus = (String)poiUtils.getCellValue(sheetIndex,rowIndex,6);

                    if(StringUtils.isEmpty(name)){
                        break;
                    }

                    Map<String, Object> searchParams = new HashMap<>();
                    searchParams.put("companyId", company.getId());
                    searchParams.put("cardNo", cardNo);
                    searchParams.put("delFlag", false);

                    List<Sort> sortList = new ArrayList<>();
                    Page<CompanyMember> page = companyMemberService.pageSearch(searchParams, 1, 100, sortList);

                    if (page.size() > 0) {
                        sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证已导入!");
                        failCount++;
                        continue;
                    } else {
                        companyMember.setId(UUID.randomUUID().toString());
                        if (StringUtils.isNotEmpty(company.getId())) companyMember.setCompanyId(company.getId());
                        if (StringUtils.isNotEmpty(name)) companyMember.setName(name);

                        CheckIdCard cic = null;

                        if(StringUtils.isEmpty(cardNo)){
                            sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证不能为空!");
                            failCount++;
                            continue;
                        }
                        else{
                            cardNo = cardNo.trim().toUpperCase();
                        }

                        for(DataDictionary dd : ddList){
                            if(dd.getName().equals(cardType)) {
                                companyMember.setCardType(dd.getValue());
                                break;
                            }
                        }

                        if (StringUtils.isEmpty(companyMember.getCardType())){
                            sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("证件类型未选择!");
                            failCount++;
                            continue;
                        }

                        if (cardType.contains("身份证")) {
                            cic = new CheckIdCard(cardNo);

                            if (!cic.validate()){
                                sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("身份证验证失败!");
                                failCount++;

                                continue;
                            }
                        }

                        companyMember.setCardNo(cardNo);

                        if (StringUtils.isNotEmpty(sex)) {
                            if (sex.contains("男")) {
                                companyMember.setSex(true);
                            } else {
                                companyMember.setSex(false);
                            }
                        } else if (cic != null && cic.validate()) {
                            companyMember.setSex(cic.isFemal());
                        }

                        if (cic != null && cic.validate()) {
                            companyMember.setAge(cic.getAge());
                        }

                        String jobsName = "";
                        if (jobName.contains("一类职业")) {
                            jobsName = "一类职业";
                        } else if (jobName.contains("二类职业")) {
                            jobsName = "二类职业";
                        } else if (jobName.contains("三类职业")) {
                            jobsName = "三类职业";
                        }

                        if (StringUtils.isEmpty(jobsName)){
                            sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("岗位/工种未选择!");
                            failCount++;
                            continue;
                        }

                        List<Jobs> jobsList = jobsService.findByName(jobsName);

                        if (jobsList.size() > 0) {
                            companyMember.setJobsId(jobsList.get(0).getId());
                        }

                        if (StringUtils.isNotEmpty(healthStatus)) companyMember.setHealthStatus(healthStatus);

                        companyMember.setStatus("0");
                        companyMember.setDelFlag(false);
                        companyMember.setCreateBy(principal.getName());
                        companyMember.setCreateTime(new Date());

                        if (companyMemberService.insert(companyMember) > 0) {
                            affectCount++;
                        }
                    }
                }
                catch(Exception innerEx){
                    logger.error(innerEx.getMessage(),innerEx);
                }
            }

            if (failCount>0){
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                int rowIndex = 1;

                while(rowIndex!= sheet1.getLastRowNum()){
                    Cell cell = sheet1.getRow(rowIndex).getCell(validateColIndex);

                    if (cell==null || StringUtils.isEmpty(cell.getStringCellValue())){
                        sheet1.removeRow(sheet1.getRow(rowIndex));
                        sheet1.shiftRows(rowIndex+1,sheet1.getLastRowNum(),-1); //删除后下面行上移，则不需要移动rowIndex
                    }
                    else {
                        rowIndex++;
                    }
                }

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                poiUtils.getWorkbook().write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig,"import","error.xls",input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
            }
            else{
                msgResult.setResult(true);
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
