package com.jpsoft.picc.modules.auth.controller;

import cn.hutool.core.convert.Convert;
import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyMember;
import com.jpsoft.picc.modules.base.service.CompanyMemberService;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.business.entity.ApplicationPolicy;
import com.jpsoft.picc.modules.business.entity.InsuranceApplication;
import com.jpsoft.picc.modules.business.entity.InsurancePolicy;
import com.jpsoft.picc.modules.business.entity.InsurancePolicyMember;
import com.jpsoft.picc.modules.business.service.ApplicationPolicyService;
import com.jpsoft.picc.modules.business.service.InsuranceApplicationService;
import com.jpsoft.picc.modules.business.service.InsurancePolicyMemberService;
import com.jpsoft.picc.modules.business.service.InsurancePolicyService;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.POIUtils;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@Api(description="每月参保人员")
@RestController
@RequestMapping("/auth/insurancePolicyUser")
public class InsurancePolicyUserController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyMemberService companyMemberService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @ApiOperation(value="读取每月参保人员分页列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applicationId",value = "投保单编号（首次投保用）", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "policyId",value = "每月投保单编号（不传policyId则查首月）", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> pageList(
            @RequestParam(value="applicationId",defaultValue="") String applicationId,
            @RequestParam(value="policyId",defaultValue="") String policyId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
//        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<Map> msgResult = new MessageResult<>();

        if (StringUtils.isNotEmpty(applicationId)) {
            List<ApplicationPolicy> relatePolicyList = applicationPolicyService.findByApplicationId(applicationId);

            if (relatePolicyList.size() == 0) {
                InsurancePolicy insurancePolicy = new InsurancePolicy();
                insurancePolicy.setId(policyId);
                insurancePolicy.setLatestApplicationId(applicationId);

                insurancePolicy.setDelFlag(false);
                insurancePolicy.setCreateBy(principal.getName());
                insurancePolicy.setCreateTime(new Date());
                insurancePolicy.setNo(1);

                int affectCount = insurancePolicyService.insert(insurancePolicy);

                if (affectCount > 0) {
                    policyId = insurancePolicy.getId();

                    ApplicationPolicy applicationPolicy = new ApplicationPolicy();
                    applicationPolicy.setId(UUID.randomUUID().toString());

                    applicationPolicy.setApplicationId(applicationId);
                    applicationPolicy.setPolicyId(policyId);

                    applicationPolicyService.insert(applicationPolicy);
                }
            }
            else if(StringUtils.isEmpty(policyId)){
                policyId = relatePolicyList.get(0).getPolicyId(); //不传policyId则查首月
            }
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","asc"));
        Map<String,Object> searchParams = new HashMap<>();

        searchParams.put("policyId",policyId);

        Page<InsurancePolicyMember> page = insurancePolicyMemberService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="读取可参保人员列表-新增投保用")
    @PostMapping("loadCompanyMemberList")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保单ID", required = true, paramType = "form",dataType = "String")
     })
    public MessageResult<List<CompanyMember>> loadCompanyMemberList(HttpServletRequest request,String policyId){
        MessageResult<List<CompanyMember>> msgResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            Company company = companyService.findByCreateBy(principal.getName());

            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

            List<String> excludePolicyList = new ArrayList<>();

            //todo 排除投保单号相同 (下月新增投保与原投保单号相同)
            excludePolicyList.add(insurancePolicy.getPolicyNo());

//            String applicationId = insurancePolicy.getLatestApplicationId();

//            if (StringUtils.isNotEmpty(applicationId)) {
//                InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);
//
//                excludePolicyIds.add(policyId);
//
//                if (StringUtils.isNotEmpty(insuranceApplication.getCombinePolicyId())){
//                    excludePolicyIds.add(insuranceApplication.getCombinePolicyId());
//                }
//            }

            //新增投保用
            List<CompanyMember> memberList = companyMemberService.findByNotInPolicyMember(company.getId(),excludePolicyList);

            msgResult.setData(memberList);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="添加参保人员")
    @PostMapping("add")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保单ID", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "memberIds",value = "企业用户ID,用,分隔", required = true, paramType = "form",dataType = "String"),
    })
    public MessageResult<String> add(@RequestParam(value="policyId",defaultValue="") String policyId,
                                                    @RequestParam(value="memberIds",defaultValue="") String memberIds,
                                                    HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<String> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;
            String[] memberIdArr = memberIds.split(",");

            for(String memberId : memberIdArr) {
                if(StringUtils.isNotEmpty(memberId) && companyMemberService.exist(memberId)) {
                    InsurancePolicyMember insurancePolicyMember = new InsurancePolicyMember();

                    insurancePolicyMember.setId(UUID.randomUUID().toString());
                    insurancePolicyMember.setPolicyId(policyId);
                    insurancePolicyMember.setMemberId(memberId);
                    insurancePolicyMember.setStatus("0");

                    insurancePolicyMember.setDelFlag(false);
                    insurancePolicyMember.setCreateBy(principal.getName());
                    insurancePolicyMember.setCreateTime(new Date());

                    Integer count = insurancePolicyMemberService.insert(insurancePolicyMember);

                    if (count > 0) {
                        affectCount++;
                    }
                }
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(policyId);
                msgResult.setMessage("新增"+affectCount+"条数据。");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除参保人员")
    @PostMapping("delete")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Integer> delete(@RequestParam(value="id",defaultValue="") String id,
                                         HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicyMember insurancePolicyMember = insurancePolicyMemberService.get(id);
            insurancePolicyMember.setDelFlag(true);
            insurancePolicyMember.setUpdateBy(principal.getName());
            insurancePolicyMember.setUpdateTime(new Date());

            int affectCount = insurancePolicyMemberService.update(insurancePolicyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="批量导入参保人员")
    @PostMapping("batchImport")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保单ID", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> batchImport(
                                             @RequestParam(value="policyId",defaultValue="") String policyId,
                                            MultipartFile uploadFile,
                                            HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<String> msgResult = new MessageResult<>();
        CompanyMember companyMember = new CompanyMember();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            List<Object[]> list = poiUtils.getAllData(0);
            list.remove(0);

            int affectCount = 0;
            int existCount = 0;
            for(Object[] array : list){
                if(StringUtils.isEmpty(array[1].toString())){
                    continue;
                }

                String name = array[1].toString();
                String cardType = array[2].toString();
                String cardNo = array[3].toString();
                String sex = array[4].toString();
                int age = Convert.toInt(array[5]);
                String jobsId = array[6].toString();
                String healthStatus = array[7].toString();

                Map<String,Object> searchParams = new HashMap<>();
                searchParams.put("delFlag",false);
                searchParams.put("companyId",company.getId());
                searchParams.put("cardNo",cardNo);
                List<Sort> sortList = new ArrayList<>();
                Page<CompanyMember> page = companyMemberService.pageSearch(searchParams,1,100,sortList);

                if(page.size() > 0) {
                    companyMember = page.get(0);
                }else{
                    companyMember.setId(UUID.randomUUID().toString());
                    if(StringUtils.isNotEmpty(company.getId())) companyMember.setCompanyId(company.getId());
                    if (StringUtils.isNotEmpty(name)) companyMember.setName(name);
                    if (StringUtils.isNotEmpty(cardType)) companyMember.setCardType(cardType);
                    if (StringUtils.isNotEmpty(cardNo)) companyMember.setCardNo(cardNo);
                    if(sex.contains("男")){
                        companyMember.setSex(false);
                    }else {
                        companyMember.setSex(true);
                    }
                    if (age != 0) companyMember.setAge(age);
                    if (StringUtils.isNotEmpty(jobsId)) companyMember.setJobsId(jobsId);
                    if (StringUtils.isNotEmpty(healthStatus)) companyMember.setHealthStatus(healthStatus);
//                if(StringUtils.isNotEmpty(cardFiles)) companyMember.setCardFiles(cardFiles);
                    companyMember.setStatus("0");
                    companyMember.setDelFlag(false);
                    companyMember.setCreateBy(principal.getName());
                    companyMember.setCreateTime(new Date());
                    companyMemberService.insert(companyMember);
                    existCount++;
                }

                Map<String,Object> searchParams1 = new HashMap<>();
                searchParams1.put("policyId",policyId);
                searchParams1.put("memberId",companyMember.getId());
                List<Sort> sortList1 = new ArrayList<>();
                Page<InsurancePolicyMember> page1 = insurancePolicyMemberService.pageSearch(searchParams1,1,100,sortList1);
                if(page1.size() <= 0) {
                    InsurancePolicyMember insurancePolicyMember = new InsurancePolicyMember();
                    insurancePolicyMember.setId(UUID.randomUUID().toString());
                    if (StringUtils.isNotEmpty(policyId)) insurancePolicyMember.setPolicyId(policyId);
                    insurancePolicyMember.setMemberId(companyMember.getId());
                    insurancePolicyMember.setStatus("0");
                    insurancePolicyMember.setDelFlag(false);
                    insurancePolicyMember.setCreateBy(principal.getName());
                    insurancePolicyMember.setCreateTime(new Date());

                    if (insurancePolicyMemberService.insert(insurancePolicyMember) > 0) {
                        affectCount++;
                    }
                }
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData("成功导入" + affectCount + "人，其中" + existCount + "人是新人，需在企业人员中完善信息。");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="替换参保人员")
    @PostMapping("replace")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyMemberId",value = "参保人员表主键", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "srcMemberId",value = "被替换企业人员表主键", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "destMemberId",value = "替换企业人员表主键", required = true, paramType = "form",dataType = "String"),
    })
    public MessageResult<String> replace(String policyMemberId,String srcMemberId,String destMemberId,
                                     HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<String> msgResult = new MessageResult<>();

        try {
            InsurancePolicyMember insurancePolicyMember = insurancePolicyMemberService.get(policyMemberId);

            insurancePolicyMember.setReplaceMemberId(srcMemberId);
            insurancePolicyMember.setMemberId(destMemberId);
            insurancePolicyMember.setStatus("0"); //替换后设为未审核
            insurancePolicyMember.setUpdateBy(principal.getName());
            insurancePolicyMember.setUpdateTime(new Date());

            Integer affectCount = insurancePolicyMemberService.update(insurancePolicyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setMessage("替换"+affectCount+"条数据。");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
